;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x4ddf4bb0012a2fdb7fa1b23554ebe92e . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd2100f7a46e11f82a53abae3c65b3b07 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0f7e7fa5907b299d237d61152e1e2cd3 . "./code.cg2js/cg-toolbar.fasl")
(#xd3bfeec7c9dbfab42928efc889971759 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xbd4b9e3bf89be6af0ff5aff162d2d78a . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x0754bc8652fa26102a8600c2284f782f . "./code.cg2js/cg-class-grid.fasl")
(#x99c8cf905bc963a4902c4687a3546e9f . "./code.cg2js/cg-tray.fasl")
(#xb6ee5a9a3acfad68c8a3d9dc1740d39d . "./code.cg2js/cg-gesture-emulation.fasl")
(#xd717b8bab34b8323f1871a37649abb29 . "./code.cg2js/cg-class-support.fasl")
(#x847e45a923d4d2c2b646925e44e920b5 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x3d8486a67061d0f4b019bdafe625e1e2 . "./code.cg2js/cg-dropping-outline.fasl")
(#xdbeeaf84cdd4e87bd08623ffba03b22b . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xad00d7cecf0f5f2a12338074afc09e56 . "./code.cg2js/cg-choice-list.fasl")
(#x0bb22896f0b7f8dc59aeb3a3f2cf5442 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7461ed466f850baad450625413568dbb . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa7ddc7d001259ea1d5fe8a00c5213304 . "./code.cg2js/cg-timer.fasl")
(#xfb751f2f3751682cd5fb04e8ae6ba452 . "./code.cg2js/cg-directory-dialog.fasl")
(#x8fdf5f5f230e79ded50b9cfbef3b4cb2 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x155bae97b424729a9d462cf4eda025dd . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x5bafb65a25fcb325739c4d16451bfa5f . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x950cff529f25c0bfad0bbbd68a5eca0c . "./code.cg2js/cg-html-widget.fasl")
(#xf5a8e2ad1a6cddb27b396773f0fd46d4 . "./code.cg2js/cg-selected-object.fasl")
(#x7f112ad276212b3adbbb2dffeb6a4fbc . "./code.cg2js/cg-lamp.fasl")
(#x507ebe77b225d61525a3fc8f1f54886a . "./code.cg2js/cg-scroll-bar.fasl")
(#xcfa466bc2bbc4508e47f75e13fa5a693 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x1aa6b65fc1c52a88d14ae260b1ab8280 . "./code.cg2js/cg-message-dialog.fasl")
(#x02259bc1d74d290787c5c62fb2ff9295 . "./code.cg2js/cg-dde-utils.fasl")
(#x3fd7fd28598e91247f9c54df88554d2e . "./code.cg2js/cg-group-box.fasl")
(#x286ebd573c736256e994c2ec1e452f73 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x131eb5b8feea5434cda047f0c281fe4d . "./code.cg2js/cg-split-bar.fasl")
(#x4ab81c7d81263c90d4ee064066a164a4 . "./code.cg2js/cg-radio-button.fasl")
(#xdcf307e75a56be61488d4a37e1aa8ea4 . "./code.cg2js/cg-base.fasl")
(#xd86cd7ba0e91149c64b03c2c3419b887 . "./code.cg2js/cg-paren-matching.fasl")
(#x8f2cff1415bb8e85f6729a042a2ac55f . "./code.cg2js/cg-graphics-context.fasl")
(#xe74c26ab5873076b76ec5617bcfd08ee . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x51ef4be2a63174f183201e630d289ae2 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x16546a4aec23290bcbe999bb8ffb1e10 . "./code.cg2js/cg-plot-widget.fasl")
(#x8bcc862a8e1e7bbe340ff29d8c0d2ba6 . "./code.cg2js/cg-dev.fasl")
(#xe642bdc516d7d388f5a6702becff6989 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xdd437f949ac4c3b45585a55c665c3275 . "./code.cg2js/cg-tab-control.fasl")
(#x90ee16dd90ca34d0d6ea13051ce6de78 . "./code.cg2js/cg-fill-texture.fasl")
(#xa8badedac9ee5b9d4432436ac83b95c5 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xd2d0d25f5d92fca5f2cca694b3e33347 . "./code.cg2js/cg-trackbar.fasl")
(#x6b16abd1ab223ef326817a1a3ca8121f . "./code.cg2js/cg-nodes.fasl")
(#xef23643188b49e83f3d8480b33487d32 . "./code.cg2js/cg-common-control.fasl")
(#xe3d32f4d54d7190220cfd3acf336c7a7 . "./code.cg2js/cg-list-view.fasl")
(#xe57e0994451cbc80021d00c62541b0bc . "./code.cg2js/cg-rich-edit.fasl")
(#x04b7e42efb3f183512790e94d779a462 . "./code.cg2js/cg-project-window.fasl")
(#xa336b67d838d49e0d900d4efd7a203d8 . "./code.cg2js/cg-item-list.fasl")
(#xf3b1a1d3930e95a593913593a176396b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xd5b900a33375bf9c42d02c045d14376d . "./code.cg2js/cg-get-position.fasl")
(#x588e98eefd500fa7691225c56136de95 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1aa730961f2f9967c39fb3a25b4ea1ff . "./code.cg2js/cg-drawable.fasl")
(#xdb147fdb07351015c02957532636d78c . "./code.cg2js/cg-lettered-menu.fasl")
(#xc1704b716a9d04666fcd32232de994cb . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x07bfb75fe4976489ac912c0de20a5981 . "./code.cg2js/cg-html-dialog.fasl")
(#xed5ed0b1a9c2cbcff6c722b0034cf682 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x2f944b1b50158de28e0b115997583ac9 . "./code.cg2js/cg-utility-dialog.fasl")
(#x2bfa65a8fa5d23bf682cb7d1eaaf006d . "./code.cg2js/cg-pixmap-widget.fasl")
(#x09e4609184683b438f744d914a45ab1c . "./code.cg2js/cg-property.fasl")
(#x58623ad6dd85b294bc3d0ebbeb5b9ab1 . "./code.cg2js/cg-menu.fasl")
(#x02abc52331b68cbc2e042fdd62896377 . "./code.cg2js/cg-clipboard.fasl")
(#xc22da54d65236f13fb8e9793a15f148c . "./code.cg2js/cg-toggling-widget.fasl")
(#xe25cbd32d0da48938f78699ad5cbf21e . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x9b9a192f54ef520d1ae95eee2044bfb5 . "./code.cg2js/cg-template-string.fasl")
(#xf8ebcaf2a6c5df8032b6a98e3e928f16 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd983074f4105265f4d77fdf3d4e3640c . "./code.cg2js/cg-os-window.fasl")
(#xe39d1750e1ec5619109510968df26f11 . "./code.cg2js/cg-grid-widget.fasl")
(#xdb3db6faab1a50a8eb5ae804d0020103 . "./code.cg2js/cg-picture-widget.fasl")
(#x9ec833df1a1d23fa64d8defcd05d9f8c . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd2d5c6207440e039bcdb50d488622963 . "./code.cg2js/cg-static-text.fasl")
(#x38660ae875e4a4be367cdf5e8d6a7da4 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x2dfb696e461265d08bcec626c54983f5 . "./code.cg2js/cg-text-widget.fasl")
(#x29e3088401b411770aa1d2177e545125 . "./code.cg2js/cg-calendar.fasl")
(#xf166817220fb86b53ead4b19509434f9 . "./code.cg2js/cg-check-box.fasl")
(#x6532a9d19b2094a28d17d6730f23106c . "./code.cg2js/cg-acache.fasl")
(#x0046237bffcfc6c9bbe0ca7096bddcdf . "./code.cg2js/cg-wrap-string.fasl")
(#x0a578bd5b5f5c7dcd6a89a5523618e84 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x7c6ba026d06a55e8e1233d8d42008d09 . "./code.cg2js/cg-font-dialog.fasl")
(#xe95368c02b73178441a54a909262bf8a . "./code.cg2js/cg-outline.fasl")
(#xf5c7a7403dea87a6a45ed9d887afad59 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xbf247ce8566d0746b41320d3b232cf3e . "./code.cg2js/cg-status-bar.fasl")
(#xbc5b647c7d31b793de5f4d71e5f71213 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x1bab391c6b2b7bd65dd63506799288bf . "./code.cg2js/cg-hotspot.fasl")
(#x83c39b14011a6e3db6c2985e63a6be6e . "./code.cg2js/cg-scaling-stream.fasl")
(#x9fd6aa4512b9ab133eb42d3b0607a41b . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9ebb740eb47176db787cc49f7baf22df . "./code.cg2js/cg-file-dialog.fasl")
(#x5ac4d8ef4ffd76b8e7876c413e1b31ba . "./code.cg2js/cg-color-dialog.fasl")
(#x4fe508f1ddc165f8d83c74a0c23b3566 . "./code.cg2js/cg-get-pixmap.fasl")
(#xd044d540158a80a1b1fb9028b2945690 . "./code.cg2js/cg-object-editor.fasl")
(#xba91c29f54e2d66ec92c27330929b702 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x96536fb69dfa33a9bf5a98ceea6e8fb9 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xec445489c34165c46a2580bb7c1fd43d . "./code.cg2js/cg-pixmap.fasl")
(#xccb71c8279c52b5a8e4abdf997be3042 . "./code.cg2js/cg-intersect.fasl")
(#x7e6d7d930a42b3f12a94a449f8de6628 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe6d4ad002e48b1987132003e5016decb . "./code.cg2js/cg-icon.fasl")
(#xc9ebcd4f9f378fb0837841fc3b0cee20 . "./code.cg2js/cg-yes-no-string.fasl")
(#x762353c60ebbb43b25754056ff2dc980 . "./code.cg2js/cg-text-or-combo.fasl")
(#x8ed7c8623387cdd52a17065fa8c0a3fa . "./code.cg2js/cg.fasl")
(#x0bc7cd3f58414e6fc0034f1e2d5b0f93 . "./code.cg2js/cg-dialog-item.fasl")
(#xd5bf1da652ceec672ebfa644ec35c40d . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa359c329c7cb70c19f122ca983ddff98 . "./code.cg2js/cg-caret.fasl")
(#x750210f9fde57c713ec3faff5c448546 . "./code.cg2js/cg-editable-text.fasl")
(#xde9b1143d36fa124853749f5d1574526 . "./code.cg2js/cg-choose-printer.fasl")
(#x3eb944f3b7e8c4beafad0115acd69e13 . "./code.cg2js/cg-yes-no-list.fasl")
(#xbefcbb2b538a9545ec30af241028c715 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd6cd44887b04c424fabc7d2d735429f1 . "./code.cg2js/cg-tooltip.fasl")
(#x1b2831a29847ce321f3338b5fbd40d69 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5712d2ef8267e7f4da3e761c0267cd14 . "./code.cg2js/cg-combo-box.fasl")
(#x08b205ba4d05dc070a6aed78275a22fd . "./code.cg2js/cg-curve.fasl")
(#xe446e38f211bc225865597bc93ac1b7e . "./code.cg2js/cg-web-browser.fasl")
(#xb9999f1ea3955916d9a20e5bfe12f0c0 . "./code.cg2js/cg-printing.fasl")
(#x19dbee3c8e59f7bad1f73285d1235ca2 . "./code.cg2js/cg-lisp-widget.fasl")
(#x0b75d5e608725f622b7cd9a5e64bd4d4 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbfb3fa53e3020b2411166ef798c65b0d . "./code.cg2js/cg-palette.fasl")
(#x88c3a5b79a0959dac1de5f5269b89a4a . "./code.cg2js/cg-os-widget.fasl")
(#xaf69e60748fa3f4c429b6dad30290386 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xaad0b032cfa853498f05d81a0c80ab5e . "./code.cg2js/cg-pixmap-printing.fasl")
(#xa17b421cf0ad4976097ea54360b53730 . "./code.cg2js/cg-button.fasl")
(#x9e145f9c9edcdb04f097ba571e9946f4 . "./code.cg2js/cg-edit-in-place.fasl")
(#x8cb9924e7e28bbf7231981ebd5669177 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xc12eb014cf42db01313c8039778f4640 . "./code.cg2js/cg-comtab.fasl")
(#xc9579ae4d1d48020b5c386be0c99ad8a . "./code.cg2js/cg-string-dialog.fasl")
(#x40e58b792d237f7120419a4b180508e1 . "./code.cg2js/cg-header-control.fasl")
(#x7eb31bd3a1c8c424bb57203bdb4b41d3 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x4c86adbe6e01f8e369a1c99a8d8cce86 . "./code.cg2js/cg-up-down-control.fasl")
(#xd7b76e6c91338e23efac98683282ccc8 . "./code.cg2js/cg-layout.fasl")
(#xcc61d78df3c57a8e64a52a60abf6f535 . "./code.cg2js/cg-chart-widget.fasl")
(#x6f8b3d39304016406a0c02a5f102dcfd . "./code.cg2js/cg-lisp-text.fasl")
(#x167809fd5cd5796766398d9bfe6f3301 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xddbe30e0a389d90115f2118f8ead6b46 . "./code.cg2js/cg-progress-indicator.fasl")
(#xf01e6d9e4a3da04298e0e9577279f4b8 . "./code.idejs/ide-inspector.fasl")
(#x61c77e0bb27c49ac4102ac4ae9b2588e . "./code.idejs/ide-grapher.fasl")
(#x825d1bdf1f3a8767ade2152a7b61da36 . "./code.idejs/ide-base.fasl")
(#x1a6bee55419a5f9570f05397bb0383f7 . "./code.idejs/ide-keyshort.fasl")
(#x45317bc5a308f2f20a8e991299bb395d . "./code.idejs/ide-dev.fasl")
(#x45f398ed7ae4a673e84f321ce73ace48 . "./code.idejs/ide-class-browser.fasl")
(#x5c430903d997f01e4f0f3997be8129f7 . "./code.idejs/ide-package-list.fasl")
(#x005cda9f9fa47d21fa5bed3b1fa44c3b . "./code.idejs/ide-intro.fasl")
(#xa1e043eb5ee51d40efd879a195a07705 . "./code.idejs/ide-class-grapher.fasl")
(#xfc19b35418a7a728d2c969e6ce2557e6 . "./code.idejs/ide-window-list.fasl")
(#xd83de8edc938e0ebab8085e941456c8f . "./code.idejs/ide-find-definitions.fasl")
(#x9c8fbd873c133d5480b4cf909f8fcdcf . "./code.idejs/ide-trace.fasl")
(#x355624f0272e84d77dd82271deb69f87 . "./code.idejs/ide-editor.fasl")
(#xa3d5d28a2e6ebe42092d02ecf1d066cc . "./code.idejs/ide-patch-dialog.fasl")
(#x9d61d905ac483ace0510f087ec4d0d34 . "./code.idejs/ide-project.fasl")
(#x7dd5cc9dc84a1b0dfb93cefa7212c3e3 . "./code.idejs/ide.fasl")
(#x57a4af49c81b8e641929e442f55faac5 . "./code.idejs/ide-apropos.fasl")
(#x3d83407bfd0f36d85d57c02e38f0d5f3 . "./code.idejs/ide-acache.fasl")
(#x57e4ae1e8f929ff83a027471bd68b788 . "./code.idejs/ide-clipboard.fasl")
(#x539f7f044534e3a998920a41c594ef1b . "./code.idejs/ide-process-browser.fasl")
(#xa0487b73e0799875615db0a5f2612a41 . "./code.idejs/ide-find-in-files.fasl")
(#x9c75fa19c0ba248c21698a46337e572b . "./code.idejs/ide-form.fasl")
(#xae378b5f0fc498123557ed2f568c2aa8 . "./code.idejs/ide-navigator.fasl")
(#xeb5b70ef6a4ced03becd92b82d3b0061 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
